@@ -1,11 +1,11 @@
 package App::Cpan;
 
-use 5.008;
+use 5.006;
 use strict;
 use warnings;
 use vars qw($VERSION);
 
-$VERSION = '1.61_03';
+$VERSION = '1.59';
 
 =head1 NAME
 
@@ -22,9 +22,6 @@ App::Cpan - easily interact with CPAN from the command line
 	# use local::lib
 	cpan -I module_name [ module_name ... ]
 
-	# one time mirror override for faster mirrors
-	cpan -p ...
-
 	# with just the dot, install from the distribution in the
 	# current directory
 	cpan .
@@ -137,11 +134,6 @@ List the modules by the specified authors.
 
 Make the specified modules.
 
-=item -M mirror1,mirror2,...
-
-A comma-separated list of mirrors to use for just this run. The C<-P>
-option can find them for you automatically.
-
 =item -n
 
 Do a dry run, but don't actually install anything. (unimplemented)
@@ -152,12 +144,11 @@ Show the out-of-date modules.
 
 =item -p
 
-Ping the configured mirrors and print a report
+Ping the configured mirrors
 
 =item -P
 
-Find the best mirrors you could be using and use them for the current
-session.
+Find the best mirrors you could be using (but doesn't configure them just yet)
 
 =item -r
 
@@ -253,7 +244,7 @@ BEGIN { # most of this should be in methods
 use vars qw( @META_OPTIONS $Default %CPAN_METHODS @CPAN_OPTIONS  @option_order
 	%Method_table %Method_table_index );
 
-@META_OPTIONS = qw( h v V I g G M: C A D O l L a r p P j: J w T);
+@META_OPTIONS = qw( h v V I g G C A D O l L a r p P j: J w T);
 
 $Default = 'default';
 
@@ -291,8 +282,6 @@ sub GOOD_EXIT () { 0 }
 	J =>  [ \&_dump_config,       NO_ARGS, GOOD_EXIT, 'Dump configuration to stdout' ],
 	F =>  [ \&_lock_lobotomy,     NO_ARGS, GOOD_EXIT, 'Turn off CPAN.pm lock files'  ],
 	I =>  [ \&_load_local_lib,    NO_ARGS, GOOD_EXIT, 'Loading local::lib'           ],
-	M =>  [ \&_use_these_mirrors,    ARGS, GOOD_EXIT, 'Setting per session mirrors'  ],
-	P =>  [ \&_find_good_mirrors, NO_ARGS, GOOD_EXIT, 'Finding good mirrors'         ],
     w =>  [ \&_turn_on_warnings,  NO_ARGS, GOOD_EXIT, 'Turning on warnings'          ],
     T =>  [ \&_turn_off_testing,  NO_ARGS, GOOD_EXIT, 'Turning off testing'          ],
 
@@ -309,6 +298,7 @@ sub GOOD_EXIT () { 0 }
 	L =>  [ \&_show_author_mods,     ARGS, GOOD_EXIT, 'Showing author mods'          ],
 	a =>  [ \&_create_autobundle, NO_ARGS, GOOD_EXIT, 'Creating autobundle'          ],
 	p =>  [ \&_ping_mirrors,      NO_ARGS, GOOD_EXIT, 'Pinging mirrors'              ],
+	P =>  [ \&_find_good_mirrors, NO_ARGS, GOOD_EXIT, 'Finding good mirrors'         ],
 
 	r =>  [ \&_recompile,         NO_ARGS, GOOD_EXIT, 'Recompiling'                  ],
 	u =>  [ \&_upgrade,           NO_ARGS, GOOD_EXIT, 'Running `make test`'          ],
@@ -373,7 +363,7 @@ sub _process_setup_options
 			);
 		}
 
-	foreach my $o ( qw(F I w T P M) )
+	foreach my $o ( qw(F I w T) )
 		{
 		next unless exists $options->{$o};
 		$Method_table{$o}[ $Method_table_index{code} ]->( $options->{$o} );
@@ -439,7 +429,7 @@ sub run
 
 		my( $sub, $takes_args, $description ) =
 			map { $Method_table{$option}[ $Method_table_index{$_} ] }
-			qw( code takes_args description );
+			qw( code takes_args );
 
 		unless( ref $sub eq ref sub {} )
 			{
@@ -459,8 +449,7 @@ sub run
 	}
 
 {
-package
-  Local::Null::Logger; # hide from PAUSE
+package Local::Null::Logger;
 
 sub new { bless \ my $x, $_[0] }
 sub AUTOLOAD { 1 }
@@ -473,7 +462,6 @@ sub _init_logger
 
     unless( $log4perl_loaded )
         {
-        print "Loading internal null logger. Install Log::Log4perl for logging messages\n";
         $logger = Local::Null::Logger->new;
         return $logger;
         }
@@ -529,22 +517,17 @@ sub _default
 		};
 
 	# How do I handle exit codes for multiple arguments?
-	my @errors = ();
+	my $errors = 0;
 
 	foreach my $arg ( @$args )
 		{
 		_clear_cpanpm_output();
 		$action->( $arg );
 
-		my $error = _cpanpm_output_indicates_failure();
-		push @errors, $error if $error;
+		$errors += defined _cpanpm_output_indicates_failure();
 		}
 
-	return do {
-		if( @errors ) { $errors[0] }
-		else { HEY_IT_WORKED }
-		};
-
+	$errors ? I_DONT_KNOW_WHAT_HAPPENED : HEY_IT_WORKED;
 	}
 
 # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
@@ -586,20 +569,11 @@ sub _clear_cpanpm_output { $scalar = '' }
 
 sub _get_cpanpm_output   { $scalar }
 
-# These are lines I don't care about in CPAN.pm output. If I can
-# filter out the informational noise, I have a better chance to
-# catch the error signal
 my @skip_lines = (
 	qr/^\QWarning \(usually harmless\)/,
 	qr/\bwill not store persistent state\b/,
 	qr(//hint//),
 	qr/^\s+reports\s+/,
-	qr/^Try the command/,
-	qr/^\s+$/,
-	qr/^to find objects/,
-	qr/^\s*Database was generated on/,
-	qr/^Going to read/,
-	qr|^\s+i\s+/|,    # the i /Foo::Whatever/ line when it doesn't know
 	);
 
 sub _get_cpanpm_last_line
@@ -630,16 +604,13 @@ sub _get_cpanpm_last_line
 
 BEGIN {
 my $epic_fail_words = join '|',
-	qw( Error stop(?:ping)? problems force not unsupported
-		fail(?:ed)? Cannot\s+install );
+	qw( Error stop(?:ping)? problems force not unsupported fail(?:ed)? );
 
 sub _cpanpm_output_indicates_failure
 	{
 	my $last_line = _get_cpanpm_last_line();
 
 	my $result = $last_line =~ /\b(?:$epic_fail_words)\b/i;
-	return A_MODULE_FAILED_TO_INSTALL if $last_line =~ /\b(?:Cannot\s+install)\b/i;
-
 	$result || ();
 	}
 }
@@ -784,12 +755,11 @@ Stolen from File::Path::Expand
 
 sub _expand_filename
 	{
-    my( $path ) = @_;
-    no warnings 'uninitialized';
-    $logger->debug( "Expanding path $path\n" );
+    my( $path )= @_;
+    print STDERR "\tExpanding $path\n";
     $path =~ s{\A~([^/]+)?}{
-		_home_of( $1 || $> ) || "~$1"
-    	}e;
+		_home_of( $1 || $> ) || ${^MATCH}
+    	}pe;
     return $path;
 	}
 
@@ -839,6 +809,7 @@ sub _is_pingable_scheme {
 sub _find_good_mirrors {
 	require CPAN::Mirrors;
 
+	my $mirrors = CPAN::Mirrors->new;
 	my $file = do {
 		my $file = 'MIRRORED.BY';
 		my $local_path = File::Spec->catfile(
@@ -851,10 +822,11 @@ sub _find_good_mirrors {
 			$local_path;
 			}
 		};
-	my $mirrors = CPAN::Mirrors->new( $file );
+
+	$mirrors->parse_mirrored_by( $file );
 
 	my @mirrors = $mirrors->best_mirrors(
-		how_many   => 5,
+		how_many   => 3,
 		verbose    => 1,
 		);
 
@@ -863,9 +835,6 @@ sub _find_good_mirrors {
 		_print_ping_report( $mirror->http );
 		}
 
-	$CPAN::Config->{urllist} = [
-		map { $_->http } @mirrors
-		];
 	}
 
 sub _print_inc_dir_report
@@ -931,19 +900,6 @@ sub _load_local_lib # -I
 	return HEY_IT_WORKED;
 	}
 
-sub _use_these_mirrors # -M
-	{
-	$logger->debug( "Setting per session mirrors" );
-	unless( $_[0] ) {
-		$logger->die( "The -M switch requires a comma-separated list of mirrors" );
-		}
-
-	$CPAN::Config->{urllist} = [ split /,/, $_[0] ];
-
-	$logger->debug( "Mirrors are @{$CPAN::Config->{urllist}}" );
-
-	}
-
 sub _create_autobundle
 	{
 	$logger->info(
@@ -1318,10 +1274,8 @@ sub _eval_version
 	{
 	my( $line, $sigil, $var ) = @_;
 
-        # split package line to hide from PAUSE
 	my $eval = qq{
-		package
-                  ExtUtils::MakeMaker::_version;
+		package ExtUtils::MakeMaker::_version;
 
 		local $sigil$var;
 		\$$var=undef; do {
@@ -1341,7 +1295,7 @@ sub _eval_version
 sub _path_to_module
 	{
 	my( $inc, $path ) = @_;
-	return if length $path < length $inc;
+	return if length $path< length $inc;
 
 	my $module_path = substr( $path, length $inc );
 	$module_path =~ s/\.pm\z//;
@@ -1383,10 +1337,14 @@ correctly if Log4perl is not installed.
 * When I capture CPAN.pm output, I need to check for errors and
 report them to the user.
 
+* Support local::lib
+
 * Warnings switch
 
 * Check then exit
 
+* ping mirrors support
+
 * no test option
 
 =head1 BUGS
@@ -1406,23 +1364,21 @@ This code is in Github:
 
 =head1 CREDITS
 
-Japheth Cleaver added the bits to allow a forced install (C<-f>).
+Japheth Cleaver added the bits to allow a forced install (-f).
 
 Jim Brandt suggest and provided the initial implementation for the
 up-to-date and Changes features.
 
-Adam Kennedy pointed out that C<exit()> causes problems on Windows
+Adam Kennedy pointed out that exit() causes problems on Windows
 where this script ends up with a .bat extension
 
-David Golden helps integrate this into the C<CPAN.pm> repos.
-
 =head1 AUTHOR
 
 brian d foy, C<< <bdfoy@cpan.org> >>
 
 =head1 COPYRIGHT
 
-Copyright (c) 2001-2013, brian d foy, All Rights Reserved.
+Copyright (c) 2001-2009, brian d foy, All Rights Reserved.
 
 You may redistribute this under the same terms as Perl itself.
 
